require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame

    class Game
        attr_reader :title

        def initialize(title)
            @title = title
            @players = []
        end
        
        def save_high_scores(to_file = "high_scores.txt")
            File.open(to_file, "w") do |file|
                file.puts "#{@title} High Scores:"
                @players.sort.each do |player|
                    file.puts high_score_entry(player)
                end
            end
        end

        def load_player(from_file)
            File.readlines(from_file).each do |line|
                add_player(Player.from_csv(line))
            end

        end

        def high_score_entry(player)
            "#{player.name.ljust(20, ".")}#{player.score}"

        # formatted_name = player.name.ljust(20, ".")
        # "#{formatted_name} #{player.score}"

        end

        def add_player(new_player)

            @players.push(new_player)

        end

        def print_stats
            puts "\n#{@title} Statistics:"
            strong_player, wimpy_player = @players.partition {|player| player.strong?} 

            # another way to do this is below...
            # strong_player = @players.select { |player| player.strong? }
            # wimpy_player = @players.reject { |player| player.strong? }

            
            puts "\n#{strong_player.size} strong players:"
            strong_player.each do |player|
                print_name_and_health(player)
            end

            puts "\n#{wimpy_player.size} wimpy players:"
            wimpy_player.each do |player|
                print_name_and_health(player)
            end

            puts "\n#{total_points} total treasure points found."

            @players.each do |player|
                puts "\n#{player.name} point totals:"
                player.each_found_treasure do |treasure|
                    puts "#{treasure.points} total #{treasure.name} points. "
                end
                puts "#{player.points} grand total points"
            end

            puts "#{title} High Scores:"
            @players.sort.each do |player|
                puts high_score_entry(player)
            end
        end

        def print_name_and_health(x)
            puts "#{x.name} (#{x.health})"
        end

        def play(rounds)
            puts "\nThere are #{@players.size} players in #{@title}:"
            @players.each do |x|
                puts x
            end

            treasures = TreasureTrove::TREASURES
            puts "There are #{treasures.size} treasures to be found:\n"
            treasures.each do |treasure|
                puts "A #{treasure.name} is worth #{treasure.points} points\n"
            end

            1.upto(rounds) do |round|
                puts "\n Round #{round}:"
                @players.each do |player|
                    GameTurn.take_turn(player)
                end
            end
        end
        
        def total_points
            @players.reduce(0) {|sum, player| sum + player.points}
        end

    end

end